@@ -1,14 +1,24 @@
-# import os
-# os.environ["EPICS_CA_AUTO_ADDR_LIST"] = "NO"
-# os.environ["EPICS_CA_ADDR_LIST"] = "129.129.122.255 sls-x12sa-cagw.psi.ch:5836"
-# os.environ["PYTHONIOENCODING"] = "latin1"
+# pylint: disable=wrong-import-position
+# pylint: disable=wrong-import-order
+# we need to run the startup script before we import anything else. This is
+# to ensure that the epics environment variables are set correctly.
+
+try:
+    from bec_plugins.device_server import startup
+except ImportError:
+    startup = None
+
+if startup is not None:
+    startup.run()
+
 
 import argparse
 import threading
 
-import device_server
 from bec_lib.core import RedisConnector, ServiceConfig, bec_logger
 
+import device_server
+
 logger = bec_logger.logger
 bec_logger.level = bec_logger.LOGLEVEL.INFO
 
