@@ -176,6 +176,8 @@ class DeviceServer(BECService):
                 self._stage_device(instructions)
             elif action == "unstage":
                 self._unstage_device(instructions)
+            elif action == "pre_scan":
+                self._pre_scan(instructions)
             else:
                 logger.warning(f"Received unknown device instruction: {instructions}")
         except ophyd_errors.LimitError as limit_error:
@@ -225,9 +227,12 @@ class DeviceServer(BECService):
         if not is_serializable(res):
             if isinstance(res, ophyd.StatusBase):
                 return res
+            elif isinstance(res, list) and instr_params.get("func") in ["stage", "unstage"]:
+                return [obj._staged for obj in res]
             res = None
             self.connector.raise_alarm(
                 severity=Alarms.WARNING,
+                alarm_type="TypeError",
                 source=instr_params,
                 content=f"Return value of rpc call {instr_params} is not serializable.",
                 metadata={},
@@ -257,6 +262,10 @@ class DeviceServer(BECService):
                     "done": res.done,
                     "settle_time": res.settle_time,
                 }
+            elif isinstance(res, list) and isinstance(res[0], ophyd.Staged):
+                res = [str(stage) for stage in res]
+            else:
+                print(f"return value: {res}")
             # send result to client
             self.producer.set(
                 MessageEndpoints.device_rpc(instr_params.get("rpc_id")),
@@ -281,17 +290,19 @@ class DeviceServer(BECService):
             sys.stdout = save_stdout
 
     def _send_rpc_exception(self, exc: Exception, instr: BECMessage.DeviceInstructionMessage):
+        exc_formatted = {
+            "error": exc.__class__.__name__,
+            "msg": exc.args,
+            "traceback": traceback.format_exc(),
+        }
+        logger.info(f"Received exception: {exc_formatted}, {exc}")
         instr_params = instr.content.get("parameter")
         self.producer.set(
             MessageEndpoints.device_rpc(instr_params.get("rpc_id")),
             BECMessage.DeviceRPCMessage(
                 device=instr.content["device"],
                 return_val=None,
-                out={
-                    "error": exc.__class__.__name__,
-                    "msg": exc.args,
-                    "traceback": traceback.format_exc(),
-                },
+                out=exc_formatted,
                 success=False,
             ).dumps(),
         )
@@ -304,7 +315,9 @@ class DeviceServer(BECService):
         for dev in devices:
             obj = self.device_manager.devices.get(dev)
             obj.metadata = instr.metadata
-            obj.obj.trigger()
+            status = obj.obj.trigger()
+            status.__dict__["instruction"] = instr
+            status.add_callback(self._status_callback)
 
     def _kickoff_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
         logger.debug(f"Kickoff device: {instr}")
@@ -320,10 +333,24 @@ class DeviceServer(BECService):
         status.add_callback(self._status_callback)
 
     def _complete_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
-        obj = self.device_manager.devices.get(instr.content["device"]).obj
-        status = obj.complete()
-        status.__dict__["instruction"] = instr
-        status.add_callback(self._status_callback)
+        if instr.content["device"] is None:
+            devices = [dev.name for dev in self.device_manager.devices.enabled_devices]
+        else:
+            devices = instr.content["device"]
+            if not isinstance(devices, list):
+                devices = [devices]
+        for dev in devices:
+            obj = self.device_manager.devices.get(dev).obj
+            if not hasattr(obj, "complete"):
+                # if the device does not have a complete method, we assume that it is done
+                status = ophyd.StatusBase()
+                status.obj = obj
+                status.set_finished()
+            else:
+                logger.debug(f"Completing device: {dev}")
+                status = obj.complete()
+            status.__dict__["instruction"] = instr
+            status.add_callback(self._status_callback)
 
     def _set_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
         device_obj = self.device_manager.devices.get(instr.content["device"])
@@ -339,6 +366,24 @@ class DeviceServer(BECService):
         status.__dict__["instruction"] = instr
         status.add_callback(self._status_callback)
 
+    def _pre_scan(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+        devices = instr.content["device"]
+        if not isinstance(devices, list):
+            devices = [devices]
+        pipe = self.producer.pipeline()
+        for dev in devices:
+            obj = self.device_manager.devices.get(dev)
+            obj.metadata = instr.metadata
+            if hasattr(obj.obj, "pre_scan"):
+                obj.obj.pre_scan()
+            dev_msg = BECMessage.DeviceReqStatusMessage(
+                device=dev,
+                success=True,
+                metadata=instr.metadata,
+            ).dumps()
+            self.producer.set_and_publish(MessageEndpoints.device_req_status(dev), dev_msg, pipe)
+        pipe.execute()
+
     def _status_callback(self, status):
         pipe = self.producer.pipeline()
         if hasattr(status, "device"):
