@@ -1,4 +1,5 @@
 import inspect
+import time
 import traceback
 from functools import reduce
 
@@ -20,6 +21,12 @@ from ophyd.signal import EpicsSignalBase
 from device_server.devices.config_update_handler import ConfigUpdateHandler
 from device_server.devices.device_serializer import get_device_info
 
+try:
+    from bec_plugins import devices as plugin_devices
+except ImportError:
+    plugin_devices = None
+
+
 logger = bec_logger.logger
 
 
@@ -70,9 +77,21 @@ class DeviceManagerDS(DeviceManagerBase):
         )
         super().initialize(bootstrap_server)
 
+    def _reload_action(self) -> None:
+        for dev, obj in self.devices.items():
+            try:
+                obj.obj.destroy()
+            except Exception:
+                logger.warning(f"Failed to destroy {obj.obj.name}")
+                raise RuntimeError
+        self.devices.flush()
+        self._get_config()
+
     def _get_device_class(self, dev_type):
         module = None
-        if hasattr(ophyd, dev_type):
+        if hasattr(plugin_devices, dev_type):
+            module = plugin_devices
+        elif hasattr(ophyd, dev_type):
             module = ophyd
         elif hasattr(opd, dev_type):
             module = opd
@@ -157,10 +176,13 @@ class DeviceManagerDS(DeviceManagerBase):
             device_classes.append(ophyd.Signal)
         if issubclass(dev_cls, EpicsSignalBase):
             device_classes.append(EpicsSignalBase)
+        if issubclass(dev_cls, ophyd.OphydObject):
+            device_classes.append(ophyd.OphydObject)
+
+        # get all init parameters of the device class and its parents
         class_params = set()
         for device_class in device_classes:
             class_params.update(inspect.signature(device_class)._parameters)
-
         class_params_and_config_keys = class_params & config.keys()
 
         init_kwargs = {key: config.pop(key) for key in class_params_and_config_keys}
@@ -168,8 +190,12 @@ class DeviceManagerDS(DeviceManagerBase):
         if device_access or (device_access is None and config.get("device_mapping")):
             init_kwargs["device_manager"] = self
 
+        signature = inspect.signature(dev_cls)
+        if "device_manager" in signature.parameters:
+            init_kwargs["device_manager"] = self
+
         # initialize the device object
-        obj = dev_cls(**init_kwargs, **config)
+        obj = dev_cls(**init_kwargs)
         self.update_config(obj, config)
 
         # refresh the device info
@@ -209,6 +235,8 @@ class DeviceManagerDS(DeviceManagerBase):
             obj.subscribe(self._obj_callback_done_moving, event_type="done_moving", run=False)
         if "flyer" in obj.event_types:
             obj.subscribe(self._obj_flyer_callback, event_type="flyer", run=False)
+        if "progress" in obj.event_types:
+            obj.subscribe(self._obj_progress_callback, event_type="progress", run=False)
         if hasattr(obj, "motor_is_moving"):
             obj.motor_is_moving.subscribe(self._obj_callback_is_moving, run=opaas_obj.enabled)
 
@@ -322,12 +350,13 @@ class DeviceManagerDS(DeviceManagerBase):
         max_points = min(len(d) for d in data.values())
         bundle = BECMessage.BundleMessage()
         for ii in range(emitted_points, max_points):
+            timestamp = time.time()
             signals = {}
             for key, val in data.items():
-                signals[key] = {"value": val[ii]}
+                signals[key] = {"value": val[ii], "timestamp": timestamp}
             bundle.append(
                 BECMessage.DeviceMessage(
-                    signals=signals,
+                    signals={obj.name: signals},
                     metadata={"pointID": ii, **metadata},
                 ).dumps()
             )
@@ -341,3 +370,13 @@ class DeviceManagerDS(DeviceManagerBase):
             MessageEndpoints.device_progress(obj.root.name), msg.dumps(), pipe=pipe
         )
         pipe.execute()
+
+    def _obj_progress_callback(self, *_args, obj, value, max_value, done, **kwargs):
+        metadata = self.devices[obj.root.name].metadata
+        msg = BECMessage.ProgressMessage(
+            value=value,
+            max_value=max_value,
+            done=done,
+            metadata=metadata,
+        )
+        self.producer.set_and_publish(MessageEndpoints.device_progress(obj.root.name), msg.dumps())
